# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["ModerationCreateResponse", "Usage"]


class Usage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int


class ModerationCreateResponse(BaseModel):
    id: str

    created: int

    detected: float

    model: str

    passed: float

    categories: Optional[List[object]] = None

    system_fingerprint: Optional[str] = None

    usage: Optional[Usage] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.



from ..._models import BaseModel

__all__ = ["ChatCompletionToolCallFunction"]


class ChatCompletionToolCallFunction(BaseModel):
    arguments: str

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_delta import ChatCompletionDelta

__all__ = ["ChatCompletionStreamChoice"]


class ChatCompletionStreamChoice(BaseModel):
    delta: ChatCompletionDelta

    index: int

    finish_reason: Optional[Literal["stop", "length", "content_filter", "tool_calls"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .completion import (
    CompletionResource,
    AsyncCompletionResource,
    CompletionResourceWithRawResponse,
    AsyncCompletionResourceWithRawResponse,
    CompletionResourceWithStreamingResponse,
    AsyncCompletionResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def completion(self) -> CompletionResource:
        return CompletionResource(self._client)

    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        return ChatResourceWithStreamingResponse(self)


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def completion(self) -> AsyncCompletionResource:
        return AsyncCompletionResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        return AsyncChatResourceWithStreamingResponse(self)


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

    @cached_property
    def completion(self) -> CompletionResourceWithRawResponse:
        return CompletionResourceWithRawResponse(self._chat.completion)


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

    @cached_property
    def completion(self) -> AsyncCompletionResourceWithRawResponse:
        return AsyncCompletionResourceWithRawResponse(self._chat.completion)


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

    @cached_property
    def completion(self) -> CompletionResourceWithStreamingResponse:
        return CompletionResourceWithStreamingResponse(self._chat.completion)


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

    @cached_property
    def completion(self) -> AsyncCompletionResourceWithStreamingResponse:
        return AsyncCompletionResourceWithStreamingResponse(self._chat.completion)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .moderation_create_params import ModerationCreateParams as ModerationCreateParams
from .moderation_create_response import ModerationCreateResponse as ModerationCreateResponse




# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_tool_call import ChatCompletionToolCall

__all__ = ["ChatCompletionMessage"]


class ChatCompletionMessage(BaseModel):
    role: Literal["assistant"]

    content: Optional[str] = None

    tool_calls: Optional[List[ChatCompletionToolCall]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ModerationCreateParams"]


class ModerationCreateParams(TypedDict, total=False):
    input: Required[str]

    model: Required[Literal["cygnet-v0"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = [
    "CompletionCreateParams",
    "Message",
    "ToolChoice",
    "ToolChoiceToolChoice",
    "ToolChoiceToolChoiceFunction",
    "Tool",
    "ToolFunction",
]


class CompletionCreateParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]

    model: Required[Literal["cygnet-v0.2"]]

    frequency_penalty: Optional[float]

    max_tokens: Optional[int]

    n: Optional[int]

    stream: Optional[bool]

    temperature: Optional[float]

    tool_choice: Optional[ToolChoice]

    tools: Optional[Iterable[Tool]]

    top_p: Optional[float]


class Message(TypedDict, total=False):
    content: Required[Optional[str]]

    role: Required[Literal["system", "user", "assistant", "data"]]


class ToolChoiceToolChoiceFunction(TypedDict, total=False):
    name: Required[str]


class ToolChoiceToolChoice(TypedDict, total=False):
    function: Required[ToolChoiceToolChoiceFunction]

    type: Required[str]


ToolChoice = Union[Literal["none", "auto"], ToolChoiceToolChoice]


class ToolFunction(TypedDict, total=False):
    description: Required[str]

    name: Required[str]

    parameters: Required[Dict[str, object]]


class Tool(TypedDict, total=False):
    function: Required[ToolFunction]

    type: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_delta_tool_call_function import ChatCompletionDeltaToolCallFunction

__all__ = ["ChatCompletionDeltaToolCall"]


class ChatCompletionDeltaToolCall(BaseModel):
    index: int

    id: Optional[str] = None

    function: Optional[ChatCompletionDeltaToolCallFunction] = None

    type: Optional[Literal["function"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_choice import ChatCompletionChoice
from .chat_completion_stream_choice import ChatCompletionStreamChoice

__all__ = [
    "CompletionCreateResponse",
    "CreateChatCompletionResponse",
    "CreateChatCompletionResponseUsage",
    "CreateChatCompletionStreamResponse",
]


class CreateChatCompletionResponseUsage(BaseModel):
    completion_tokens: int

    prompt_tokens: int

    total_tokens: int


class CreateChatCompletionResponse(BaseModel):
    id: str

    choices: List[ChatCompletionChoice]

    created: int

    model: str

    object: Literal["chat.completion"]

    system_fingerprint: Optional[str] = None

    usage: Optional[CreateChatCompletionResponseUsage] = None


class CreateChatCompletionStreamResponse(BaseModel):
    id: str

    choices: List[ChatCompletionStreamChoice]

    created: int

    model: str

    object: Literal["chat.completion.chunk"]

    system_fingerprint: Optional[str] = None


CompletionCreateResponse = Union[CreateChatCompletionResponse, CreateChatCompletionStreamResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_tool_call_function import ChatCompletionToolCallFunction

__all__ = ["ChatCompletionToolCall"]


class ChatCompletionToolCall(BaseModel):
    id: str

    function: ChatCompletionToolCallFunction

    type: Literal["function"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_message import ChatCompletionMessage

__all__ = ["ChatCompletionChoice"]


class ChatCompletionChoice(BaseModel):
    finish_reason: Literal["stop", "length", "content_filter", "tool_calls"]

    index: int

    message: ChatCompletionMessage

    logprobs: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chat_completion_delta_tool_call import ChatCompletionDeltaToolCall

__all__ = ["ChatCompletionDelta"]


class ChatCompletionDelta(BaseModel):
    content: Optional[str] = None

    role: Optional[Literal["system", "user", "assistant", "tool"]] = None

    tool_calls: Optional[List[ChatCompletionDeltaToolCall]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .completion import (
    CompletionResource,
    AsyncCompletionResource,
    CompletionResourceWithRawResponse,
    AsyncCompletionResourceWithRawResponse,
    CompletionResourceWithStreamingResponse,
    AsyncCompletionResourceWithStreamingResponse,
)

__all__ = [
    "CompletionResource",
    "AsyncCompletionResource",
    "CompletionResourceWithRawResponse",
    "AsyncCompletionResourceWithRawResponse",
    "CompletionResourceWithStreamingResponse",
    "AsyncCompletionResourceWithStreamingResponse",
    "ChatResource",
    "AsyncChatResource",
    "ChatResourceWithRawResponse",
    "AsyncChatResourceWithRawResponse",
    "ChatResourceWithStreamingResponse",
    "AsyncChatResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .chat_completion_delta import ChatCompletionDelta as ChatCompletionDelta
from .chat_completion_choice import ChatCompletionChoice as ChatCompletionChoice
from .chat_completion_message import ChatCompletionMessage as ChatCompletionMessage
from .completion_create_params import CompletionCreateParams as CompletionCreateParams
from .chat_completion_tool_call import ChatCompletionToolCall as ChatCompletionToolCall
from .completion_create_response import (
    CreateChatCompletionResponseUsage as CreateChatCompletionResponseUsage,
    CreateChatCompletionResponse as CreateChatCompletionResponse,
    CreateChatCompletionStreamResponse as CreateChatCompletionStreamResponse,
    CompletionCreateResponse as CompletionCreateResponse
)
from .chat_completion_stream_choice import ChatCompletionStreamChoice as ChatCompletionStreamChoice
from .chat_completion_delta_tool_call import ChatCompletionDeltaToolCall as ChatCompletionDeltaToolCall
from .chat_completion_tool_call_function import ChatCompletionToolCallFunction as ChatCompletionToolCallFunction
from .chat_completion_delta_tool_call_function import (
    ChatCompletionDeltaToolCallFunction as ChatCompletionDeltaToolCallFunction,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...types.chat import CreateChatCompletionResponse, CreateChatCompletionStreamResponse, completion_create_params
from ..._base_client import (
    make_request_options,
)
from ...types.chat.completion_create_response import CompletionCreateResponse

__all__ = ["CompletionResource", "AsyncCompletionResource"]


class CompletionResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CompletionResourceWithRawResponse:
        return CompletionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CompletionResourceWithStreamingResponse:
        return CompletionResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Literal["cygnet-v0.2"],
        frequency_penalty: Optional[float] | NotGiven = NOT_GIVEN,
        max_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        n: Optional[int] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        tool_choice: Optional[completion_create_params.ToolChoice] | NotGiven = NOT_GIVEN,
        tools: Optional[Iterable[completion_create_params.Tool]] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateChatCompletionResponse | Stream[CreateChatCompletionStreamResponse]:
        """
        Completions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
                "/chat/completion",
                body=maybe_transform(
                    {
                        "messages": messages,
                        "model": model,
                        "frequency_penalty": frequency_penalty,
                        "max_tokens": max_tokens,
                        "n": n,
                        "stream": stream,
                        "temperature": temperature,
                        "tool_choice": tool_choice,
                        "tools": tools,
                        "top_p": top_p,
                    },
                    completion_create_params.CompletionCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=CreateChatCompletionResponse,
                stream=stream or False,
                stream_cls=Stream[CreateChatCompletionStreamResponse]
            )


class AsyncCompletionResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCompletionResourceWithRawResponse:
        return AsyncCompletionResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCompletionResourceWithStreamingResponse:
        return AsyncCompletionResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        messages: Iterable[completion_create_params.Message],
        model: Literal["cygnet-v0.2"],
        frequency_penalty: Optional[float] | NotGiven = NOT_GIVEN,
        max_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        n: Optional[int] | NotGiven = NOT_GIVEN,
        stream: Optional[bool] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        tool_choice: Optional[completion_create_params.ToolChoice] | NotGiven = NOT_GIVEN,
        tools: Optional[Iterable[completion_create_params.Tool]] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateChatCompletionStreamResponse | AsyncStream[CreateChatCompletionStreamResponse]:
        """
        Completions

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
                "/chat/completion",
                body=await async_maybe_transform(
                    {
                        "messages": messages,
                        "model": model,
                        "frequency_penalty": frequency_penalty,
                        "max_tokens": max_tokens,
                        "n": n,
                        "stream": stream,
                        "temperature": temperature,
                        "tool_choice": tool_choice,
                        "tools": tools,
                        "top_p": top_p,
                    },
                    completion_create_params.CompletionCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=CreateChatCompletionResponse,
                stream=stream or False,
                stream_cls=AsyncStream[CreateChatCompletionStreamResponse]
            )


class CompletionResourceWithRawResponse:
    def __init__(self, completion: CompletionResource) -> None:
        self._completion = completion

        self.create = to_raw_response_wrapper(
            completion.create,
        )


class AsyncCompletionResourceWithRawResponse:
    def __init__(self, completion: AsyncCompletionResource) -> None:
        self._completion = completion

        self.create = async_to_raw_response_wrapper(
            completion.create,
        )


class CompletionResourceWithStreamingResponse:
    def __init__(self, completion: CompletionResource) -> None:
        self._completion = completion

        self.create = to_streamed_response_wrapper(
            completion.create,
        )


class AsyncCompletionResourceWithStreamingResponse:
    def __init__(self, completion: AsyncCompletionResource) -> None:
        self._completion = completion

        self.create = async_to_streamed_response_wrapper(
            completion.create,
        )
